﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Syncfusion.Windows.Forms.Diagram;
using Syncfusion.Windows.Forms.Diagram.Controls;
using System.ComponentModel;
using System.Data;
using System.Drawing;

namespace Syncfusion.Diagram.WindowsExt
{
    public class DiagramExt : Syncfusion.Windows.Forms.Diagram.Controls.Diagram
    {
        public DiagramExt(DiagramController ext)
            : base()
        {
            this.Model = ext.Model;
        }

        private int i = 0;
        private object dataSource;
        private string dataMember;
        private string dataFieldChild;
        private string dataFieldParent;
        private Node dataNode;

        [TypeConverter("System.Windows.Forms.Design.DataSourceConverter, System.Design")]
        [Category("Data")]
        [DefaultValue(null)]
        public object DataSource
        {
            get
            {
                return this.dataSource;
            }
            set
            {
                if (this.dataSource != value)
                {
                    this.dataSource = value;
                    DataBinding();
                }
            }
        }

        [Category("Data")]
        [Editor("System.Windows.Forms.Design.DataMemberListEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing")]
        [DefaultValue("")]
        public string DataMember
        {
            get
            {
                return this.dataMember;
            }
            set
            {
                if (this.dataMember != value)
                {
                    this.dataMember = value;
                    DataBinding();
                }
            }
        }

        [Category("Data")]
        [Editor("System.Windows.Forms.Design.DataMemberListEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing")]
        [DefaultValue("")]
        public string DataFieldChild
        {
            get
            {
                return this.dataFieldChild;
            }
            set
            {
                if (this.dataFieldChild != value)
                {
                    this.dataFieldChild = value;
                }
            }
        }

        [Category("Data")]
        [Editor("System.Windows.Forms.Design.DataMemberListEditor, System.Design", "System.Drawing.Design.UITypeEditor, System.Drawing")]
        [DefaultValue("")]
        public string DataFieldParent
        {
            get
            {
                return this.dataFieldParent;
            }
            set
            {
                if (this.dataFieldParent != value)
                {
                    this.dataFieldParent = value;
                }
            }
        }

        public Node DataNode
        {
            get
            {
                return this.dataNode;
            }
            set
            {
                if (this.dataNode != value)
                {
                    this.dataNode = value;
                }
            }
        }

        private void DataBinding()
        {
            DataTable table = new DataTable();
            table = this.dataSource as DataTable;
            this.Model.RenderingStyle.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
            foreach (DataRow row in table.Rows)
            {
                string id = row[this.DataFieldChild].ToString();
                string parentId = row[this.DataFieldParent].ToString();
                NodePopulate(id, parentId);
            }
            foreach (DataRow row in table.Rows)
            {
                string id = row[this.DataFieldChild].ToString();
                string parentId = row[this.DataFieldParent].ToString();
                ConnectorPopulate(id, parentId);
            }
        }


        private void ConnectorPopulate(string id, string parentId)
        {
            Node pn = (Node)this.Model.Nodes.FindNodeByName(parentId);
            Node nd = (Node)this.Model.Nodes.FindNodeByName(id);

            if (pn != null)
            {
                AddConnector(pn, nd);
            }

            this.View.SelectionList.Clear();
        }

        private void AddConnector(Node pn, Node nd)
        {
            OrgLineConnector orgconn = new OrgLineConnector(new PointF(100, 200), new PointF(200, 200));
            pn.CentralPort.TryConnect(orgconn.TailEndPoint);
            nd.CentralPort.TryConnect(orgconn.HeadEndPoint);
            this.Model.AppendChild(orgconn);
        }

        private void NodePopulate(string id, string parentId)
        {
            if (id != null && parentId != null)
            {
                Node nd = this.DataNode.Clone() as Node;
                nd.Name = id;
                AddLabel(nd, id, parentId);
                this.Model.AppendChild(nd);

            }

        }

        private void AddLabel(Node nd, string id, string parentId)
        {
            PathNode nd1 = (Node)nd as PathNode;
            Syncfusion.Windows.Forms.Diagram.Label lbl = new Syncfusion.Windows.Forms.Diagram.Label();
            lbl.FontColorStyle.Color = Color.White;
            nd.Name = id;
            lbl.Text = id;
            lbl.Name = id;
            if (nd is Group)
            {
                Group set = nd as Group;
                lbl.Text = "Rectangle " + i;
                set.Labels.Add(lbl);
                i++;
            }
            else
            {
                nd1.Labels.Add(lbl);
            }
        }
    }
}
